﻿/*
 * Created by SharpDevelop.
 * User: misthema
 * Date: 14.7.2016
 * Time: 11:44
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */


				
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TradeServer.ScriptingDriver.DataObjects;
using TradeServer.ScriptingDriver.Interfaces;

namespace PshaisIndicators
{
    public class ChandelierExit : IIndicatorScript
    {
        int neededHistorryTicks = 50; // default field, used to store the needed depth
        int length = 24;
        double atrLongMult = 3;
        double atrShortMult = 3;

        double atrLong, atrShort;

        // Properties
        public string Name
        {
            get { return "Chandelier Exit"; }
        }
        public int TicksBack
        {
        	get { return length; }
            // get { return neededHistorryTicks; } // This code is disable because we can do better here
        }

        // Setting fields
        public List<ScriptParameter> GetParameters()
        {
            List<ScriptParameter> inputField = new List<ScriptParameter>
            {
                new ScriptParameter("Length", ScriptParameterType.Integer, length.ToString()),
                new ScriptParameter("Long Multiplier", ScriptParameterType.Decimal, atrLongMult.ToString()),
                new ScriptParameter("Short Multiplier", ScriptParameterType.Decimal, atrShortMult.ToString())
            };

            return inputField;
        }

        public void SetParameters(Dictionary<string, object> parameters)
        {
            foreach (var item in parameters) // lets just loop them all
            {
                if (item.Key == "Length") { length = Convert.ToInt32(item.Value); }
                if (item.Key == "Long Multiplier") { atrLongMult = ParseDouble(item.Value.ToString()); }
                if (item.Key == "Short Multiplier") { atrShortMult = ParseDouble(item.Value.ToString()); }
            }
        }

        // Chart lines
        public List<ChartRecord> GetChartLines()
        {
            List<ChartRecord> chartLines = new List<ChartRecord>
            {
                new ChartRecord() { ChartIndex = 0, Name = "Short", HexLineColor = "#FF0000" },
                new ChartRecord() { ChartIndex = 0, Name = "Long", HexLineColor = "#FFFF00" }
            };

            return chartLines;
        }
        public List<decimal> GetChartData()
        {
            List<decimal> chartData = new List<decimal>
            {
                Convert.ToDecimal(atrShort),
                Convert.ToDecimal(atrLong)
            };

            // context.Logger.LogToFile(string.Format("Called short {0} and long {1}", ma_short, ma_long));

            return chartData;
        }

        // This is called as the script is loaded the first time, aka the init
        public void Init()
        {
            // In this code example we do not have to do anything
        }

        // This is the brain so the say, this is where the calculation is done and an output must be produced.
        public IndicatorResult GetResult(IndicatorContext context)
        {
            try
            {
                IPriceInstrument instrument = context.PriceInstrument; // Its pre-created for speed based on the "TicksBack" property field
                var atrOut = new double[10];
                int beginIndex, outNBElements;
                
                int position = instrument.Close.Count() - (1 + context.Offset); // minus 1 because of counting from 0 + account for the backtest
                
                var price = instrument.Close[position];

                var macdReturnCode = TicTacTec.TA.Library.Core.Atr( position,
                                                                       position,
                                                                       instrument.High,
                                                                       instrument.Low,
                                                                       instrument.Close,
                                                                       length,
                                                                       out beginIndex,
                                                                       out outNBElements,
                                                                       atrOut
                                                                      );

                double atr = 0d;
                
                // Verification
                if (macdReturnCode == TicTacTec.TA.Library.Core.RetCode.Success)
                {
                    if (outNBElements > 0)
                    {
                        atr = atrOut[outNBElements-1];
                    }
                }
                
                // ===============================
				// Chandelier Exit start
				// -------------------------------
				double[] highestHighs = new double[22], lowestLows = new double[22], temp;
				
				/*Split<double>( high, testB+i, out highestHighs, out temp );
				Split<double>( low, testB+i, out lowestLows, out temp );
				var highestHigh = highestHighs.Max();
				var lowestLow = lowestLows.Min();*/
				
				for(int k = 0; k < 22; k++ )
				{
					highestHighs[k] = instrument.High[position - k];
					lowestLows[k] = instrument.Low[position - k];
				}
				
				var highestHigh = highestHighs.Max();
				var lowestLow = lowestLows.Min();
				
				/*var highestHigh = data.Select((value, index) => new {value, index})
					.OrderByDescending(vi => vi.value)
					.First();
				
				var lowestLow = data.Select((value, index) => new {value, index})
					.OrderBy(vi => vi.value)
					.First();*/
				
				var atrLongAdd = atr * atrLongMult;
				var atrShortAdd = atr * atrShortMult;
				atrLong = highestHigh - atrLongAdd;
				atrShort = lowestLow + atrShortAdd;
				// -------------------------------
				// Chandelier Exit end
				// ===============================

                // lets signal in this example on the cross over of the 2 moving averages
                //double diff = ma_long - ma_short;

                if( price < atrLong )
                	return IndicatorResult.SellShort;
                
                if( price > atrShort )
                	return IndicatorResult.BuyLong;
                
                return IndicatorResult.StayNoPosition;

            }
            catch (Exception ex) // This try catch block is an highly recommend practice
            {
                context.Logger.LogToFile(string.Format("Exception detected: {0}", ex.Message));
            }

            return IndicatorResult.StayNoPosition; // Default output
        }

		double ParseDouble(object obj)
		{
			var numStyle = System.Globalization.NumberStyles.Any;
			var cult = System.Globalization.CultureInfo.InvariantCulture;
			
			double output;
			try {
				double.TryParse(
					obj.ToString().Replace(',', '.'),
					numStyle,
					cult,
					out output);
			}
			catch
			{
				output = 0;
			}
			
			return output;
		}
    }
}
